/*
 * DoubleCheck, a UCI chess playing engine. Copyright (C) 2011 Lucas Braesch.
 * 
 * DoubleCheck is free software: you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * DoubleCheck is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If not,
 * see <http://www.gnu.org/licenses/>.
*/
#include <string.h>
#include "search.h"
#include "uci_option.h"
#include "util.h"

#ifdef POSIX
	#include <unistd.h>
	#include <sys/select.h>
#else
#include <windows.h>
#include <conio.h>
#define send   printf // 'send' conflicts with 'send' in windows.h/winsock.h
#endif

static Board B;

static bool loop_step();
static void parse_go();
static void parse_option(const char *line);
static void parse_position(const char *line);

void main_loop()
{
	init_bitboard();
	init_psq();
	init_zobrist();
	option_init();

	setvbuf(stdin, NULL, _IONBF, 0);
	setvbuf(stdout, NULL, _IONBF, 0);
	
	if (!uci_mode) printf("> ");

	tt_alloc(option_get_int("Hash"));

	while (loop_step());
	
	tt_free();
}

static bool loop_step()
// returns true, unless stdin pipe is closed or quit command issued
{
	char line[0x1000];
	if (!fgets(line, 0x1000, stdin))
		return false;	// pipe closed
	
	char *ptr = strchr(line,'\n');
	if (ptr) *ptr = '\0';
	const char *token = strtok(line, " ");
	
	if (!strcmp(token, "isready"))
		send("readyok\n");
	

	if (!strcmp(token, "go")) {
        parse_go();
    }
    else if (!strcmp(token, "uci")) {
		send("\nid name DoubleCheck 2.0 JA\n");
		send("id author Lucas Braesch\n");
		option_list();
		send("uciok\n");
		uci_mode = true;
	}
	else if (!strcmp(token, "setoption"))
		parse_option(line + strlen("setoption "));
	else if (!strcmp(token, "ucinewgame"))
		tt_clear();
	else if (!strcmp(token, "position"))
		parse_position(line + strlen("position "));
	
	else if (!strcmp(token, "quit"))
		return false;
	else if (!strcmp(token, "stop"))
		abort_search = true;
	else if (!strcmp(token, "eval")) {
		print_board(&B);
		send("\neval = %d", calc_eval(&B));
	}


    if (!uci_mode) printf("\n> ");
	return true;
}

static void parse_option(const char *line)
{
	const char *name = strstr(line, "name ");
	char *value = strstr(line, "value ");

	if (!name || !value || name >= value) return; // ignore buttons

	value[-1] = '\0'; // HACK: terminate the name string
	name += strlen("name ");
	value += strlen("value ");

	int old_hash = option_get_int("Hash");
	option_set(name, value);

	if (!strcmp(name, "Hash") && option_get_int("Hash") != old_hash) {
		tt_free();
		tt_alloc(option_get_int("Hash"));
	}
}

static void parse_position(const char *line)
{
	const char *fen = strstr(line, "fen ");
	const char *moves = strstr(line, "moves ");

	if (fen)
		load_fen(&B, fen + strlen("fen "));
	else	// assume startpos
		load_fen(&B, StartFen);

	if (moves) {
		const char *ptr = moves + strlen("moves ");
		char s[8];

		while (*ptr != '\0') {
			s[0] = *ptr++;
			s[1] = *ptr++;
			s[2] = *ptr++;
			s[3] = *ptr++;

			if (*ptr == '\0' || *ptr == ' ')
				s[4] = '\0';
			else {  	// promotion
				s[4] = *ptr++;
				s[5] = '\0';
			}

			move_t m = string_to_move(&B, s);
			play(&B, m);

			while (*ptr == ' ') ptr++;
		}
	}
}

static void parse_go()
{
	const char *token;
	int wtime = 0, btime = 0, winc = 0, binc = 0;
	int move_time = 0, max_depth = 0;
	U64 max_nodes = 0;

	for (token = strtok(NULL," "); token; token = strtok(NULL," ")) {
		if (!strcmp(token, "wtime")) {
			if (!(token = strtok(NULL, " ")))
				fatal("parse_go(): missing argument\n");
			wtime = atoi(token);
		} else if (!strcmp(token, "btime")) {
			if (!(token = strtok(NULL, " ")))
				fatal("parse_go(): missing argument\n");
			btime = atoi(token);
		} else if (!strcmp(token, "winc")) {
			if (!(token = strtok(NULL, " ")))
				fatal("parse_go(): missing argument\n");
			winc = atoi(token);
		} else if (!strcmp(token, "binc")) {
			if (!(token = strtok(NULL, " ")))
				fatal("parse_go(): missing argument\n");
			binc = atoi(token);
		} else if (!strcmp(token, "movetime")) {
			if (!(token = strtok(NULL, " ")))
				fatal("parse_go(): missing argument\n");
			move_time = atoi(token);
		} else if (!strcmp(token, "nodes")) {
			if (!(token = strtok(NULL, " ")))
				fatal("parse_go(): missing argument\n");
			max_nodes = atoi(token);	// TODO: is there an ISO C atoi 64 bit ?
		} else if (!strcmp(token, "depth")) {
			if (!(token = strtok(NULL, " ")))
				fatal("parse_go(): missing argument\n");
			max_depth = atoi(token);
		} else if (!strcmp(token, "infinite")) {
			move_time = 0;
			break;
		}
	}

	const int blitz_time = B.turn ? btime : wtime;
	const int blitz_inc = B.turn ? binc : winc;
	const int max_time = blitz_time + blitz_inc > 0
		? min(blitz_time/40 + blitz_inc, blitz_time-100)
		: move_time;

	UseNull = option_get_bool("NullMove");
	UseIID = option_get_bool("IID");
	UseDelta = option_get_bool("Delta");
	UseFutility = option_get_bool("Futility");
	UseLMR = option_get_bool("LMR");

	move_t best = id_loop(&B, max_time, max_nodes, max_depth);

	char s[8];
	move_to_string(&B, best, s, false);
	send("\nbestmove %s\n", s);
}

bool input_available()
{
#ifdef POSIX
	fd_set set;
	FD_ZERO(&set);
	FD_SET(STDIN_FILENO, &set);

	struct timeval time_val;
	time_val.tv_sec = 0;
	time_val.tv_usec = 0;

	return select(STDIN_FILENO+1, &set, NULL, NULL, &time_val) > 0;   
#else	

	 static int init = 0, pipe;
  static HANDLE inh;
  DWORD dw;
   /* If we're running under UCI then we can't use _kbhit() as the input commands
    * are sent to us directly over the internal pipe */
  if (uci_mode) {
#if defined(FILE_CNT)
    if (stdin->_cnt > 0) return stdin->_cnt;
#endif
    if (!init) {
      init = 1;
      inh = GetStdHandle(STD_INPUT_HANDLE);
      pipe = !GetConsoleMode(inh, &dw);
      if (!pipe) {
        SetConsoleMode(inh, dw & ~(ENABLE_MOUSE_INPUT | ENABLE_WINDOW_INPUT));
        FlushConsoleInputBuffer(inh);
      }
    }
    if (pipe) {
      if (!PeekNamedPipe(inh, NULL, 0, NULL, &dw, NULL)) return 1;
      return dw;
    }
    else {
      GetNumberOfConsoleInputEvents(inh, &dw);
      return dw <= 1 ? 0 : dw;
    }
  }
  else return _kbhit();

#endif
}

void event()
{
	while (!abort_search && input_available())
		loop_step();
}
